/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"

WOFS_IMPL int8_t wofs__file_restore_temp(WOFS_STATE_ARG
	const wofs_ino_t temp_ino)
{
	int8_t ec;

	const uint8_t shift = WOFS_STATE.blockszsh;

	uint16_t clen;
	wofs_ino_t orig_ino;

	/* before performing the restore, fetch previously stored data */

	const wofs_size_t block_size = ((wofs_size_t)1) << shift;

	const wofs_addr_t temp_addr =
		((wofs_addr_t)WOFS_INODE_TO_BLOCK(temp_ino)) << shift;

	struct wofs_inode inode;

	WOFS_READ_EXTERN(&inode, temp_addr, sizeof(inode));
	inode.i_type = WOFS_INOTYPE_FILE;

	WOFS_READ_EXTERN(&clen,
		temp_addr+block_size-sizeof(uint16_t),
		sizeof(clen));

	WOFS_READ_EXTERN(&orig_ino,
		temp_addr+block_size-sizeof(uint16_t)-sizeof(wofs_ino_t),
		sizeof(orig_ino));

	const wofs_addr_t orig_addr =
		((wofs_addr_t)WOFS_INODE_TO_BLOCK(orig_ino)) << shift;

	/* 7. Erase the original inode */

	if(0!=wroncefs_op_erase(PASS_STATE orig_addr, block_size))
	{
		return -WOFS_ERR_EIOE;
	}

	/* 8. Copy the file info, and the file content back */

	if(0!=(ec=wofs__copy_storage_content(PASS_STATE orig_addr+sizeof(inode),
		temp_addr+sizeof(inode), clen+WOFS_SIZEOF_FILEINFO)))
	{
		return ec;
	}

	/* 9. Fill the inode structure */

	if(0!=(ec=wofs__ino_fill_1(PASS_STATE &inode, orig_addr)) ||
		0!=(ec=wofs__ino_fill_2(PASS_STATE &inode, orig_addr)))
	{
		return ec;
	}

	/* 10. Erase the temporary file inode */

	if(0!=wroncefs_op_erase(PASS_STATE temp_addr, block_size))
	{
		return -WOFS_ERR_EIOE;
	}

	return 0;
}
