/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"

WOFS_IMPL int8_t wofs__compile_temp_inode(WOFS_STATE_ARG
	struct wofs_inode* const temp, const struct wofs_inode* const original,
	const uint8_t i_type)
{
	temp->i_sig   = WOFS_SIG;
	temp->i_first = original->i_first;
	temp->i_next  = original->i_next;
	temp->i_type  = i_type;
	temp->i_vnext = original->i_vnext;
	temp->i_resd  = 0xFF;
	temp->i_alive = 0xFF;

	if(WOFS_END_INODE!=temp->i_next && 0x00!=temp->i_vnext)
	{
		/* the next inode cannot be accepted as a valid chain, because */
		/* the 'i_next' is not valid (due to 'i_vnext') -- remove this */
		/* link, and leave some garbage on the filesystem that a subsequent */
		/* mount could collect */

		temp->i_next  = WOFS_END_INODE;
		temp->i_vnext = 0xFF;
	}

	return 0;
}
