/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"
#include <string.h>

static int8_t fd_read_impl(WOFS_STATE_ARG struct wofs_fd* const fd, void* buf,
	wofs_size_t size)
{
	/* this wrapper function is needed because of the nature of the */
	/* WOFS_READ_EXTERN() macro */

	WOFS_READ_EXTERN(buf,
		fd->fd_addr+sizeof(struct wofs_inode)+WOFS_SIZEOF_FILEINFO+fd->fd_lpos,
		size);

	return 0;
}

WOFS_IMPL wofs_ssize_t wofs_read(WOFS_STATE_ARG struct wofs_fd* const fd,
	void* buf, wofs_size_t size, int8_t* const out_ec)
{
	wofs_ssize_t res = 0;
	int8_t       ec  = 0;

	while(0<size)
	{
		wofs_size_t part = fd->fd_clen - fd->fd_lpos;

		if(0==part)
		{
			if(0!=(ec=wofs__follow_file_chain__forward(PASS_STATE fd)))
			{
				if((-WOFS_ERR_ENOMORE)==ec)
				{
					ec = 0;
				}

				break;
			}

			continue;
		}

		if(part>size)
		{
			part = size;
		}

		if(0!=(ec=fd_read_impl(PASS_STATE fd, buf, part)))
		{
			break;
		}

		fd->fd_lpos += part;
		fd->fd_fpos += part;
		buf = ((uint8_t*)buf) + part;
		res += part;
		size -= part;
	}

	if(NULL!=out_ec)
	{
		*out_ec = ec;
	}

	return res;
}
