/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "wroncefs.h"
#include "run.h"
#include "io.h"
#include <stdio.h>
#include <string.h>

int run_case1(const uint16_t nblocks, const uint8_t blkszsh)
{
	const wofs_size_t block_size = ((wofs_size_t)1) << blkszsh;

	wofs_ino_t wd = WOFS_ROOT_INODE;

	CHECK(0, wofs_format(nblocks, blkszsh, 0));

	wofs_ino_t ino;
	struct wofs_fd fd;

	CHECK(0, wofs_mkdir(WOFS_ROOT_INODE, "dir1", &ino));
	CHECK(0, wofs_mkdir(ino, "subdir1-1", NULL));
	CHECK(0, wofs_mkdir(ino, "subdir1-2", NULL));

	CHECK(0, wofs_mkdir(WOFS_ROOT_INODE, "dir2", &ino));
	CHECK(0, wofs_mkdir(ino, "subdir2-1", NULL));
	CHECK(0, wofs_mkdir(ino, "subdir2-2", NULL));

	CHECK(0, wofs_creat(&fd, WOFS_ROOT_INODE, "file1"));
	CHECK(12, wofs_write(&fd, "hello world!", 12, NULL));

	CHECK(0, wofs_creat(&fd, WOFS_ROOT_INODE, "file2"));
	size_t file2_size = 0;
	for(int i=1;i<=2000;++i)
	{
		char buf[16];
		const int len = sprintf(buf, "%i", i);
		CHECK(len, wofs_write(&fd, buf, len, NULL));
		CHECK(1, wofs_write(&fd, "\n", 1, NULL));
		file2_size += 1 + len;
	}

	/* remount */
	CHECK(0, wofs_mount(nblocks, blkszsh, WOFS_MOUNT_NORMAL));

	CHECK(0, wofs_open(&fd, WOFS_ROOT_INODE, "file2"));
	CHECK(file2_size, wofs_seek(&fd, 0, WOFS_SEEK_END, NULL));

	{
		char buf[4];
		CHECK(0xE6C, wofs_seek(&fd, 0xE6C, WOFS_SEEK_SET, NULL));
		CHECK(4, wofs_read(&fd, buf, sizeof(buf), NULL));
		CHECK(0, memcmp(buf, "951\n", 4));
	}

	return 0;
}
